#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const configureRollpkg_1 = require("./configureRollpkg");
const bundlephobiaStats_1 = require("./bundlephobiaStats");
const rollupBuilds_1 = require("./rollupBuilds");
const utils_1 = require("./utils");
const errorUtils_1 = require("./errorUtils");
const rollpkg = async () => {
    /////////////////////////////////////
    // clean dist folder
    const cleanDistMessage = 'Cleaning dist folder';
    try {
        const clean = utils_1.cleanDist();
        await utils_1.progressEstimator(clean, cleanDistMessage);
    }
    catch (error) {
        errorUtils_1.logError({
            failedAt: cleanDistMessage,
            message: errorUtils_1.errorAsObjectWithMessage(error).message,
            fullError: error,
        });
        throw errorUtils_1.EXIT_ON_ERROR;
    }
    /////////////////////////////////////
    /////////////////////////////////////
    // rollpkg invariants and configuration
    const args = process.argv.slice(2);
    const cwd = process.cwd();
    const invariantsAndConfigurationMessage = 'Checking rollpkg invariants';
    let rollpkgConfiguration;
    try {
        rollpkgConfiguration = configureRollpkg_1.checkInvariantsAndGetConfiguration({ args, cwd });
        await utils_1.progressEstimator(rollpkgConfiguration, invariantsAndConfigurationMessage);
    }
    catch (error) {
        errorUtils_1.logRollpkgError({
            failedAt: invariantsAndConfigurationMessage,
            message: errorUtils_1.errorAsObjectWithMessage(error).message,
        });
        throw errorUtils_1.EXIT_ON_ERROR;
    }
    const { watchMode, tsconfigPath, addUmdBuild, includeBundlephobiaStats, entryFile, kebabCasePkgName, pkgJsonSideEffects, pkgJsonDependencyKeys, pkgJsonPeerDependencyKeys, pkgJsonUmdGlobalDependencies, } = await rollpkgConfiguration;
    /////////////////////////////////////
    /////////////////////////////////////
    // create rollup config
    const rollupConfigurationMessage = 'Creating rollup config';
    let rollupConfiguration;
    try {
        rollupConfiguration = rollupBuilds_1.createRollupConfig({
            tsconfigPath,
            addUmdBuild,
            kebabCasePkgName,
            pkgJsonSideEffects,
            pkgJsonPeerDependencyKeys,
            pkgJsonUmdGlobalDependencies,
        });
        await utils_1.progressEstimator(Promise.resolve(), rollupConfigurationMessage);
    }
    catch (error) {
        errorUtils_1.logError({
            failedAt: rollupConfigurationMessage,
            message: errorUtils_1.errorAsObjectWithMessage(error).message,
            fullError: error,
        });
        throw errorUtils_1.EXIT_ON_ERROR;
    }
    const { buildPluginsDefault, buildPluginsWithNodeEnvDevelopment, buildPluginsWithNodeEnvProduction, outputPluginsDefault, outputPluginsProduction, treeshakeOptions, umdNameForPkg, umdExternalDependencies, umdDependencyGlobals, } = rollupConfiguration;
    /////////////////////////////////////
    /////////////////////////////////////
    // rollup watch
    if (watchMode) {
        rollupBuilds_1.rollupWatch({
            kebabCasePkgName,
            pkgJsonDependencyKeys,
            pkgJsonPeerDependencyKeys,
            entryFile,
            treeshakeOptions,
            buildPluginsDefault,
            outputPluginsDefault,
        });
        return;
    }
    /////////////////////////////////////
    /////////////////////////////////////
    // create rollup bundles
    const createRollupBundlesMessage = `Creating esm, cjs${addUmdBuild ? ', umd' : ''} builds`;
    let bundles;
    try {
        bundles = rollupBuilds_1.createBundles({
            entryFile,
            pkgJsonDependencyKeys,
            pkgJsonPeerDependencyKeys,
            umdExternalDependencies,
            treeshakeOptions,
            buildPluginsDefault,
            buildPluginsWithNodeEnvDevelopment,
            buildPluginsWithNodeEnvProduction,
            addUmdBuild,
        });
        await utils_1.progressEstimator(bundles, createRollupBundlesMessage, {
            id: `${kebabCasePkgName}-${createRollupBundlesMessage}`,
        });
    }
    catch (error) {
        const errorAsObject = errorUtils_1.errorAsObjectWithMessage(error);
        // rpt2 is the rollup typescript plugin
        if (errorAsObject.plugin === 'rpt2') {
            errorUtils_1.logTsError({
                failedAt: createRollupBundlesMessage,
                message: errorAsObject.message,
            });
        }
        else {
            errorUtils_1.logError({
                failedAt: createRollupBundlesMessage,
                message: errorAsObject.message,
                fullError: error,
            });
        }
        throw errorUtils_1.EXIT_ON_ERROR;
    }
    const [bundleDefault, bundleCjsProd, bundleUmdDev, bundleUmdProd,] = await bundles;
    /////////////////////////////////////
    /////////////////////////////////////
    // write rollup bundles
    const writeRollupBundlesMessage = `Writing esm, cjs${addUmdBuild ? ', umd' : ''} builds`;
    try {
        const output = rollupBuilds_1.writeBundles({
            cwd,
            kebabCasePkgName,
            bundleDefault,
            bundleCjsProd,
            bundleUmdDev,
            bundleUmdProd,
            outputPluginsDefault,
            outputPluginsProduction,
            umdNameForPkg,
            umdDependencyGlobals,
        });
        await utils_1.progressEstimator(output, writeRollupBundlesMessage, {
            id: `${kebabCasePkgName}-${writeRollupBundlesMessage}`,
        });
    }
    catch (error) {
        errorUtils_1.logError({
            failedAt: writeRollupBundlesMessage,
            message: errorUtils_1.errorAsObjectWithMessage(error).message,
            fullError: error,
        });
        throw errorUtils_1.EXIT_ON_ERROR;
    }
    /////////////////////////////////////
    /////////////////////////////////////
    // rollpkg build success!
    await utils_1.progressEstimator(Promise.resolve(), 'ROLLPKG BUILD SUCCESS 😁😘');
    /////////////////////////////////////
    /////////////////////////////////////
    // calculate bundlephobia package stats
    if (includeBundlephobiaStats) {
        const bundlephobiaStatsMessage = 'Calculating Bundlephobia stats';
        try {
            const packageStats = bundlephobiaStats_1.calculateBundlephobiaStats({ cwd });
            await utils_1.progressEstimator(packageStats, bundlephobiaStatsMessage, {
                id: `${kebabCasePkgName}-${bundlephobiaStatsMessage}`,
            });
            bundlephobiaStats_1.printBundlephobiaStats(await packageStats);
        }
        catch (error) {
            errorUtils_1.logError({
                failedAt: bundlephobiaStatsMessage,
                message: `Bundlephobia Error: ${errorUtils_1.errorAsObjectWithMessage(error).message}`,
            });
            // don' throw EXIT_ON_ERROR because the build has already succeeded
            // and an error in stats calculation shouldn't cause `rollpkg build` to fail
        }
    }
    /////////////////////////////////////
};
// always exit 0 in watch mode so can chain in npm scripts: rollpkg watch && ...
const errorExitCode = process.argv[2] === 'watch' ? 0 : 1;
rollpkg().catch((error) => {
    if (error === errorUtils_1.EXIT_ON_ERROR) {
        // known error, it has already been logged so just clean dist and exit
        utils_1.cleanDist()
            .catch()
            .finally(() => {
            process.exit(errorExitCode);
        });
    }
    else {
        // unknown error, so just log and exit (leave dist as it is to help with debugging)
        errorUtils_1.logError({ message: 'Rollpkg UNKNOWN ERROR', fullError: error });
        process.exit(errorExitCode);
    }
});
//# sourceMappingURL=cli.js.map